@@ -7,6 +7,7 @@ import hashlib
 import time
 import tarfile
 import json
+import imp
 
 from .environments import Environment
 from .exceptions import IpkgException
@@ -49,12 +50,13 @@ class Formula(NameVersionRevisionComparable):
     """
     name = None
     version = None
-    revision = None
+    revision = 1
     sources = None
     patches = tuple()
     dependencies = tuple()
     homepage = None
     envvars = None
+    build_envvars = None
     """Arguments passed to ``./configure``"""
     configure_args = ['--prefix=%(prefix)s']
 
@@ -68,8 +70,6 @@ class Formula(NameVersionRevisionComparable):
         self.environment = environment
         self.verbose = verbose
         self.log = log or logging.getLogger(__name__ )
-        self.commands = {}
-        self.__command_id = 0
         self.__cwd = os.getcwd()
 
     def run_command(self, command, data=None, cwd=None):
@@ -82,22 +82,8 @@ class Formula(NameVersionRevisionComparable):
         else:
             stdout = stderr = open(os.devnull, 'w')
 
-        start = time.time()
-        exit_code = self.environment.execute(command,
-                                             stdout=stdout, stderr=stderr,
-                                             cwd=cwd or self.__cwd, data=data),
-
-        report = {
-            'command': command,
-            'start': start,
-            'end': time.time(),
-            'exit_code': exit_code,
-            'cwd': cwd,
-        }
-
-        self.commands[self.__command_id] = report
-        self.__command_id += 1
-        return report
+        return self.environment.execute(command, stdout=stdout, stderr=stderr,
+                                        cwd=cwd or self.__cwd, data=data),
 
     def run_configure(self):
         command = ['./configure']
@@ -118,7 +104,8 @@ class Formula(NameVersionRevisionComparable):
         else:
             raise AttributeError(attr)
 
-    def build(self, package_dir, remove_build_dir=True, repository=None):
+    def build(self, package_dir, remove_build_dir=True, repository=None,
+              os_name=None, os_release=None, arch=None):
         """Build the formula."""
         LOGGER.debug('%r.build(package_dir=%s, remove_build_dir=%s)', self, package_dir, remove_build_dir)
 
@@ -132,6 +119,9 @@ class Formula(NameVersionRevisionComparable):
             self.environment = Environment(prefix)
             self.environment.directories.create()
 
+        if self.build_envvars:
+            self.environment.variables.add(self.build_envvars)
+
         env_prefix = self.environment.prefix
 
         # Install dependencies in build environment
@@ -140,7 +130,8 @@ class Formula(NameVersionRevisionComparable):
                         ', '.join(self.dependencies))
             for dependency in self.dependencies:
                 if dependency not in self.environment.packages:
-                    self.environment.install(dependency, repository)
+                    self.environment.install(dependency, repository,
+                                             os_name, os_release, arch)
                     installed_dependencies.append(dependency)
 
         # Create the sources root directory
@@ -229,11 +220,12 @@ class Formula(NameVersionRevisionComparable):
         meta_tarinfo.mode = 0644
         meta_tarinfo.size = meta_string_size
 
-        with tarfile.open(filepath, 'w:bz2') as pkg:
-            pkg.addfile(meta_tarinfo, meta_string)
-            for pkg_file in files:
-                pkg.add(os.path.join(self.environment.prefix, pkg_file),
-                        pkg_file, recursive=False)
+        pkg = tarfile.open(filepath, 'w:bz2')
+        pkg.addfile(meta_tarinfo, meta_string)
+        for pkg_file in files:
+            pkg.add(os.path.join(self.environment.prefix, pkg_file),
+                    pkg_file, recursive=False)
+        pkg.close()
 
         LOGGER.info('Package %s created', filepath)
 
@@ -250,20 +242,26 @@ class Formula(NameVersionRevisionComparable):
         """
         #LOGGER.debug('%s.from_file("%s")', cls.__name__, filepath)
 
-        globals_ = {'Formula': cls, 'File': File, 'platform': platform}
-        locals_ = {}
+        if not os.path.exists(filepath):
+            raise IpkgException('Formula not found: %s' % filepath)
 
-        with open(filepath) as bf:
-            exec bf.read() in globals_, locals_
+        filepath = os.path.abspath(filepath)
+        filename = os.path.basename(filepath)
+        module_name = filename.split('.py')[0].replace('.', '_')
+
+        module = imp.load_source(module_name, filepath)
 
         formula_classes = []
-        for obj in locals_.values():
+        for attr in dir(module):
+            if attr.startswith('_'):
+                continue
+            obj = getattr(module, attr)
             try:
-                is_formula = issubclass(obj, Formula)
+                is_formula_cls = issubclass(obj, Formula)
             except TypeError:
                 pass
             else:
-                if is_formula:
+                if is_formula_cls and obj is not Formula:
                     formula_classes.append(obj)
 
         if formula_classes:
@@ -274,6 +272,8 @@ class Formula(NameVersionRevisionComparable):
         else:
             raise IpkgException('No Formula class found')
 
+        setattr(module, formula_class.__name__, formula_class)
+
         return formula_class
 
     def __repr__(self):
