@@ -16,6 +16,7 @@ from .packages import BasePackage, MetaPackage, PackageFile
 from .prefix_rewriters import rewrite_prefix
 from .utils import DictFile, execute, make_package_spec, mkdir
 from .compat import basestring
+from .files.exceptions import FilesException
 from . import platform
 
 
@@ -162,6 +163,8 @@ class EnvironmentVariables(dict):
     """
     def __init__(self, directories, defaults=os.environ):
 
+        self.__directories = directories
+
         if isinstance(defaults, dict) or \
            defaults is os.environ:
             defaults = dict(defaults)
@@ -225,6 +228,24 @@ class EnvironmentVariables(dict):
     def __str__(self):
         return self.as_string()
 
+    def add(self, name, value=None):
+        """Add an environment variable.
+        """
+        if isinstance(name, dict):
+            variables = name
+        else:
+            variables = {name: value}
+
+        for var_name, var_value in variables.items():
+            try:
+                var_value = var_value % self.__directories
+            except KeyError:
+                # invalid format string ?
+                pass
+
+            LOGGER.debug('Adding variable %s=%s', var_name, var_value)
+            self[var_name] = Variable(var_name, var_value)
+
 
 class Environment(object):
     """An ipkg environment.
@@ -243,8 +264,8 @@ class Environment(object):
         # add their custom environment variables
         if 'packages' in self.meta:
             for name, data in self.meta['packages'].items():
-                if 'envvars' in data:
-                    self.__add_package_envvars(data['envvars'])
+                if 'envvars' in data and data['envvars'] is not None:
+                    self.variables.add(data['envvars'])
         else:
             self.meta['packages'] = {}
 
@@ -304,26 +325,30 @@ class Environment(object):
 
         LOGGER.info('Package %s uninstalled', package)
 
-    def install(self, package, repository=None):
+    def install(self, package, repository=None,
+                os_name=None, os_release=None, arch=None):
         """Install a package.
         """
         LOGGER.info('Installing %s', package)
 
+        os_name = os_name or platform.NAME
+        os_release = os_release or platform.RELEASE
+        arch = arch or platform.ARCHITECTURE
+
         if isinstance(package, basestring):
-            if os.path.exists(package):
-                # If package is a string and exists on the filesystem,
-                # use it
+
+            try:
                 package = PackageFile(package)
-            else:
+
+            except FilesException:
                 # If it does not exist, and this environment has a repository,
                 # try to find it using the repository.
                 if repository is None:
                     raise IpkgException('Cannot find package %s' % package)
                 else:
-                    package = repository.find(package,
-                                              os_name=platform.NAME,
-                                              os_release=platform.RELEASE,
-                                              arch=platform.ARCHITECTURE)
+                    package = repository.find(package, os_name=os_name,
+                                              os_release=os_release,
+                                              arch=arch)
 
         elif not isinstance(package, BasePackage):
             raise IpkgException('Invalid package: %r' % package)
@@ -350,7 +375,8 @@ class Environment(object):
             for dependency in package.dependencies:
                 if dependency not in self.meta['packages']:
                     LOGGER.info('Installing dependency: %s', dependency)
-                    self.install(dependency, repository)
+                    self.install(dependency, repository,
+                                 os_name, os_release, arch)
 
         package.extract(self.prefix)
 
@@ -372,24 +398,11 @@ class Environment(object):
         self.meta.save()
 
         # Load package custom environment variables
-        if hasattr(package, 'envvars'):
-            self.__add_package_envvars(package.envvars)
+        if package.envvars is not None:
+            self.variables.add(package.envvars)
 
         LOGGER.info('Package %s installed', make_package_spec(package))
 
-    def __add_package_envvars(self, envvars):
-        """Load package custom environment variables."""
-        if isinstance(envvars, dict):
-            for name, value in envvars.items():
-                try:
-                    value = value % self.directories
-                except KeyError:
-                    # invalid format string ?
-                    pass
-
-                LOGGER.debug('Adding variable %s=%s', name, value)
-                self.variables[name] = Variable(name, value)
-
     @property
     def packages(self):
         return map(MetaPackage, self.meta['packages'].values())
