@@ -6,10 +6,11 @@ import subprocess
 import tarfile
 import zipfile
 import errno
+import shlex
 
 from .files import vopen
 from .exceptions import IpkgException
-from .compat import basestring
+from .compat import basestring, StringIO
 from .regex import PACKAGE_SPEC
 
 
@@ -118,12 +119,12 @@ def execute(command,
         if stdin:
             kw['stdin'] = stdin
     else:
-        data = data.encode('UTF-8') # Python 3 compat
+        #data = data.encode('UTF-8') # Python 3 compat
         kw['stdin'] = PIPE
 
     # If command is a string, split it to get a format that Popen understands
     if isinstance(command, basestring):
-        command_ = command.split()
+        command_ = shlex.split(command)
     else:
         command_ = command
 
@@ -143,10 +144,10 @@ def execute(command,
         raise ExecutionFailed(command_,
                               'exited with code %i' % process.returncode)
 
-    if hasattr(stdout_str, 'decode'):
-        stdout_str = stdout_str.decode('UTF-8')
-    if hasattr(stderr_str, 'decode'):
-        stderr_str = stderr_str.decode('UTF-8')
+    #if hasattr(stdout_str, 'decode'):
+    #    stdout_str = stdout_str.decode('UTF-8')
+    #if hasattr(stderr_str, 'decode'):
+    #    stderr_str = stderr_str.decode('UTF-8')
 
     return stdout_str, stderr_str
 
@@ -191,21 +192,45 @@ def parse_package_spec(spec):
         raise InvalidPackage(spec)
 
 
+def which(command):
+    """Find a command in ``$PATH``.
+    """
+    for directory in os.environ['PATH'].split(':'):
+        path = os.path.join(directory, command)
+        if os.path.exists(path) and os.path.isfile(path):
+            return path
+
+
 def unarchive(fileobj, target):
+    """Extract an archive, detecting its format.
+
+    Supports: tar.bz2, tar.gz, tar.xz, zip
+    """
     LOGGER.debug('unarchive(%r, %r)', fileobj, target)
 
-    if fileobj.name.endswith('.tar.gz') or \
-       fileobj.name.endswith('.tar.bz2'):
+    filename = fileobj.name
+
+    # Since tarfile cannot handle xz archives,
+    # we first use the xz tool to uncompress it
+    if filename.endswith('.tar.xz'):
+        xz = which('xz')
+        if not xz:
+            raise IpkgException('Cannot find the xz tool')
+        fileobj = StringIO(
+            execute('xz -d -c', data=fileobj.read(), stdout=PIPE)[0])
+
+    if filename.split('.')[-2:] in (
+        ['tar', 'bz2'], ['tar', 'gz'], ['tar', 'xz']):
         archive = tarfile.open(fileobj=fileobj)
         root_items = set(i.path.split('/')[0] for i in archive)
 
-    elif fileobj.name.endswith('.zip'):
+    elif filename.endswith('.zip'):
         archive = zipfile.ZipFile(fileobj)
         root_items = set(i.filename.split('/')[0] for i in
                          archive.filelist)
 
     else:
-        raise IpkgException('Unrecognized file type %s' % fileobj.name)
+        raise IpkgException('Unrecognized file type %s' % filename)
 
     if len(root_items) != 1:
         raise IpkgException('There must be strictly 1 item at '
